﻿using System;
using System.IO;
using System.Windows.Forms;

namespace MVJPad
{
    
    public partial class Form1 : Form
    {
        
        private const int KEY_CODE_INDEX = 1;       // ListViewのキーを表示する位置

        public class Consts
        {
            public const int BUTTON_MAX = 16;       // ジョイパッドのボタン数（十字キー含む）
            public const int BUTTON_OFFSET = 4;     // ジョイパッドのボタン0のオフセット
            public const uint SIGN = 0x504A564D;    // Sign "MVJP"
        }
        
        struct AssignedKeys
        {
            public int[] Buttons;
        };

        private Form FKeyForm;
        private AssignedKeys FAssignedKeys;

        public Form1()
        {
            InitializeComponent();
            FAssignedKeys.Buttons = new int[Consts.BUTTON_MAX];
            InitAssignedKey();
            UpdateListView();
        }



        /// <summary>
        /// ListViewダブルクリックイベント。キー入力フォームを作成する
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listView_MouseDoubleClick(object sender, MouseEventArgs e)
        {

            if (listView.SelectedItems.Count == 0)
            {
                return;
            }
                        
            FKeyForm = new KeyDownForm();
            FKeyForm.FormBorderStyle = FormBorderStyle.None;
            FKeyForm.KeyDown += new KeyEventHandler(KeyFormOnKeyDown);
            FKeyForm.StartPosition = FormStartPosition.CenterParent;
            FKeyForm.ShowDialog();
        }

        /// <summary>
        /// 割り当て用キー入力のイベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void KeyFormOnKeyDown(object sender, KeyEventArgs e)
        {
            FKeyForm.Close();
            SetAssignedKey(listView.SelectedItems[0].Index, (int)e.KeyCode);
            SetKeyToListView(listView.SelectedItems[0], (int)e.KeyCode);
        }

        /// <summary>
        /// キー入力を割り当てる
        /// </summary>
        /// <param name="itemIndex"></param>
        /// <param name="keyCode"></param>
        private void SetAssignedKey(int itemIndex, int keyCode)
        {
            FAssignedKeys.Buttons[itemIndex] = keyCode;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            InitAssignedKey();
            UpdateListView();
        }

        /// <summary>
        /// キー割り当てをデフォルト値で初期化
        /// </summary>
        private void InitAssignedKey()
        {
            Array.Clear(FAssignedKeys.Buttons, 0, FAssignedKeys.Buttons.Length);
            FAssignedKeys.Buttons[0] = (int)Keys.Left;
            FAssignedKeys.Buttons[1] = (int)Keys.Up;
            FAssignedKeys.Buttons[2] = (int)Keys.Right;
            FAssignedKeys.Buttons[3] = (int)Keys.Down;
            FAssignedKeys.Buttons[Consts.BUTTON_OFFSET + 0] = (int)Keys.ShiftKey;
            FAssignedKeys.Buttons[Consts.BUTTON_OFFSET + 1] = (int)Keys.X;
            FAssignedKeys.Buttons[Consts.BUTTON_OFFSET + 2] = (int)Keys.Z;
            FAssignedKeys.Buttons[Consts.BUTTON_OFFSET + 3] = (int)Keys.A;
            FAssignedKeys.Buttons[Consts.BUTTON_OFFSET + 4] = (int)Keys.S;
            FAssignedKeys.Buttons[Consts.BUTTON_OFFSET + 5] = (int)Keys.D;
            FAssignedKeys.Buttons[Consts.BUTTON_OFFSET + 6] = (int)Keys.Q;
            FAssignedKeys.Buttons[Consts.BUTTON_OFFSET + 7] = (int)Keys.W;
            
        }

        /// <summary>
        /// ListViewを更新
        /// </summary>
        private void UpdateListView()
        {
            for (int i = 0; i < listView.Items.Count; i++)
            {
                ListViewItem item = listView.Items[i];

                SetKeyToListView(item, FAssignedKeys.Buttons[i]);
            }
            
        }

        /// <summary>
        /// 割り当てたキーをListViewに表示
        /// </summary>
        /// <param name="item"></param>
        /// <param name="keyCode"></param>
        private void SetKeyToListView(ListViewItem item, int keyCode)
        {
            KeyEventArgs key = new KeyEventArgs((Keys)keyCode);

            SetKeyToListView(item, key);
        }

        /// <summary>
        /// 割り当てたキーをListViewに表示
        /// </summary>
        /// <param name="item"></param>
        /// <param name="key"></param>
        private void SetKeyToListView(ListViewItem item, KeyEventArgs key)
        {
            if (item.SubItems.Count <= KEY_CODE_INDEX)
            {
                item.SubItems.Add(key.KeyCode.ToString());
            }
            else
            {
                item.SubItems[KEY_CODE_INDEX].Text = key.KeyCode.ToString();
            }
        }

        /// <summary>
        /// 設定ファイルの保存
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuCreateSettingFile_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.FileName = "MVJPad.mvj";
            sfd.Filter = "設定ファイル(*.mvj)|*.mvj|すべてのファイル(*.*)|*.*";
            sfd.RestoreDirectory = true;

            if (sfd.ShowDialog() == DialogResult.OK)
            {
                SaveSettingFile(sfd.FileName);
                MessageBox.Show("キーボード割り当てファイル \"MVJPad.mvj\" が作成されました。\nこのファイルと同封の \"xinput1_3.dll\" をゲーム本体と同じフォルダにコピーしてください。");          
            }
        }

        /// <summary>
        /// 設定ファイルの読み込み
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuOpenSettingFile_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.FileName = "MVJPad.mvj";
            ofd.Filter = "設定ファイル(*.mvj)|*.mvj|すべてのファイル(*.*)|*.*";
            ofd.RestoreDirectory = true;

            if (ofd.ShowDialog() == DialogResult.OK)
            {
                OpenSettingFile(ofd.FileName);
            }
        }

        /// <summary>
        /// キー割り当てを読み込む
        /// </summary>
        /// <param name="fileName"></param>
        private void OpenSettingFile(string fileName)
        {
            FileStream fs = new FileStream(fileName, FileMode.Open);
            BinaryReader reader = new BinaryReader(fs);

            // Signの読み込み
            UInt32 sign = reader.ReadUInt32();

            if (sign == Consts.SIGN)
            {
                fs.Seek(16, SeekOrigin.Begin);

                // ボタンデータの読み込み
                for (int i = 0; i < Consts.BUTTON_MAX; i++)
                {
                    FAssignedKeys.Buttons[i] = reader.ReadInt32();
                }
                

                UpdateListView();
            }
            else
            {
                MessageBox.Show("不正なファイル");
            }

            reader.Close();
            fs.Close();
        }

        /// <summary>
        /// キー割り当てを保存
        /// </summary>
        /// <param name="fileName"></param>
        private void SaveSettingFile(string fileName)
        {
            FileStream fs = new FileStream(fileName, FileMode.Create);
            BinaryWriter writer = new BinaryWriter(fs);

            UInt32 sign = Consts.SIGN;
            UInt32 reserved = 0;

            // Sign書き込み
            writer.Write(sign);
            writer.Write(reserved);
            writer.Write(reserved);
            writer.Write(reserved);

            // ボタンデータ書き込み
            for (int i = 0; i < Consts.BUTTON_MAX; i++)
            {
                writer.Write(FAssignedKeys.Buttons[i]);                    
            }
            

            writer.Close();
            fs.Close();
        }
        
        /// <summary>
        /// deleteボタンを押した時に割り当てを削除する
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                SetAssignedKey(listView.SelectedItems[0].Index, 0);
                SetKeyToListView(listView.SelectedItems[0], 0);
                    
            }
        }
    }
}
